#ifdef WIN32
#include <windows.h>	//needed if using windows
#endif


#include <GL/glut.h>	//functions for openGL
#include <GL/gl.h>	// Header File For The OpenGL32 Library
#include <GL/glu.h>	// Header File For The GLu32 Library
#include "vectorMath.h"	//some vector calculation functions
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#define M_PI 3.14159265358979323846
#define TRIANGLE 3
#define MAXVERTICES 50 + 2
#define MAXTRIANGLES 50
#define fileIn  "mesh.txt"

//	corner Table for T mesh
int v[MAXTRIANGLES*TRIANGLE];	//list of vector from g to triangle
int o[MAXTRIANGLES*TRIANGLE];	//list of opposite corner to g.
vertex g[MAXVERTICES];		//holds the original set of points
	
//	Flags used in program
int tol	= 3;		//tolerance distance for detecting vertex
int movingVertex  = -1;	//FLAG to tell if we're moving a vertex
int removeVertex  = -1;	//FLAG to tell if we need to remove a vertex
int fullMemory    = -1;	//FLAG to tell if we reached the max number of Triangle;

//	Important varibles
int numOfTriangles = 0;
int numOfVertices = 0;
int cornerSelect = -1;
FILE *file;


//returns the triangle number
int tOpt(int x){
	return (int)x/3;
}

int nOpt(int x){
	int tri = tOpt(x);
	return 3*tri + (x+1)%3;
}

int pOpt(int x){
	return nOpt(nOpt(x));
}

int oOpt(int x){
	if(o[x] == -1)
		return x;
	else
		return o[x];
}

int rOpt(int x){
	return oOpt(nOpt(x));
}

int lOpt(int x){
	return oOpt(pOpt(x));
}

int vOpt(int x){
	return v[x];
}

vertex gOpt(int x){
	return g[v[x]];
}

void print_triangles(){
	int i;
	for(i =0; i < numOfTriangles; i++)
		printf("tri:%d %d,%d,%d\n",i,v[3*i],v[3*i+1],v[3*i+2]);
}

void print_vertex(){
	int i;
	for(i = 0; i < numOfVertices;i++)
		toString(g[i]);
}

void print_oCorners(){
	int i;
	for(i = 0; i < numOfTriangles; i++)
		printf("tri:%d %d,%d,%d\n",i,o[3*i],o[3*i+1],o[3*i+2]);
}

int equals(vertex this, vertex that,int TOL){
	return    (this.x <= that.x + TOL && this.x >= that.x - TOL 
		&& this.y <= that.y + TOL && this.y >= that.y - TOL
	      //&& this.z <= that.z + TOL && this.z >= that.z - TOL
		   );
}

void baryBall(){
	vertex temp;

	temp = divide(add(add(g[v[cornerSelect]],g[v[nOpt(cornerSelect)]]),g[v[pOpt(cornerSelect)]]),3);
	temp = divide(sub(temp,g[v[cornerSelect]]),4);
	temp = add(temp, g[v[cornerSelect]]);

	glBegin(GL_QUADS);
		glColor3f(0.0,1.0,0.0);

		glVertex3f(temp.x-tol,temp.y-tol,0.0);
		glVertex3f(temp.x-tol,temp.y+tol,0.0);
		glVertex3f(temp.x+tol,temp.y+tol,0.0);
		glVertex3f(temp.x+tol,temp.y-tol,0.0);
	glEnd();

}

/*
 * inserts vertex at given point p in array
 * NOTE: z defaults to zero for now
 */
void insert_vertex(int x, int y){

	vertex temp;
	temp.x = x; temp.y = y; temp.z = 0.0;
	if (numOfVertices < MAXVERTICES){
		g[numOfVertices] = temp;
		numOfVertices++;
printf("vertex:%d ",numOfVertices);
toString(g[numOfVertices-1]);
	}else{
		fullMemory = 1;

	}
}

void insert_triangle(int p1, int p2, int p3){
	if(numOfTriangles < MAXTRIANGLES){
		v[3*numOfTriangles    ] = p1;
		v[3*numOfTriangles + 1] = p2;
		v[3*numOfTriangles + 2] = p3;
printf("new tri %d,%d,%d\n",p1,p2,p3);
	numOfTriangles++;
	}else{
		fullMemory = 1;
	}
}

/*
 * remove most recent triangle */
void remove_Triangle(){
	int i;
	printf("%d:%d\n",v[3*(numOfTriangles-1)], numOfVertices);

	if(numOfTriangles > 1){
		numOfTriangles -= 1;
		if(v[3*(numOfTriangles)] == numOfVertices-1){
			printf("sub\n");
			numOfVertices -= 1;	
		}
		for(i=0; i < 3; i++)	//clear out o table for current triangle//
			o[3*numOfTriangles + i] = -1;

		for(i=0; i < 3*numOfTriangles; i++){ // clear out all reference to removed triangle in o table//
			if(o[i] == 3*numOfTriangles || o[i] == 3*numOfTriangles + 1 || o[i] == 3*numOfTriangles + 2)
				o[i] = -1;
		}

		if(cornerSelect == 3*numOfTriangles || cornerSelect == 3*numOfTriangles + 1 || cornerSelect == 3*numOfTriangles + 2)
			cornerSelect = -1;
	}
}



/*
 * Checks if the clicked point is near a vertex
 */
int vertex_near_point(int x, int y){
	int i;
	vertex temp;
	temp.x = x; temp.y =y; temp.z = 0;
		for(i = 0; i < numOfVertices; i++){
			if(equals(temp, g[i], tol+20))
				return i; //return vertex
		}
	return -1;
}


//reads in vertex data from file mesh.txt
int readFile(){
	int a=0,b=0,i, temp;
	float x,y,z;
	file = fopen(fileIn,"r");

	//reset//
	cornerSelect = -1;

	numOfVertices =0;

	if(file == NULL){// file not found
		return 1;
	}

	fscanf(file,"%d",&numOfTriangles);
	fscanf(file,"%d",&temp);

printf("read 1 %d, %d\n",numOfTriangles, temp);

	for(i=0; i<numOfTriangles*3;i++){
		fscanf(file,"%d,%d",&a,&b);
		v[i] = a;
		o[i] = b;
	}

	for(i=0; i<temp;i++){
		fscanf(file,"%f,%f,%f",&x,&y,&z);
		insert_vertex((int)x,(int)y);
	}

	fclose(file);
	
	return 0;
}

/*
 *save the current graphic to vertex.txt
 */
void saveFile(){
	int i;
	file = fopen(fileIn,"w");

	fprintf(file,"%d\n",numOfTriangles);
	fprintf(file,"%d\n",numOfVertices);

	for(i=0; i<numOfTriangles*3;i++)
		fprintf(file,"%d,%d\n",v[i],o[i]);
	
	for(i=0; i<numOfVertices;i++)
		fprintf(file,"%f,%f,%f\n",g[i].x,g[i].y,g[i].z);
	

	fclose(file);

}


/*
 * Takes in a triangle and two points
 */
int edge_near_point(int tri, int x, int y){

	vertex a,b,c,d;
	float min=1e20;
	float dist,dot1,dot2;
	int i,mini=-1;

	for (i=0; i< 3; i++){
		a=g[v[3*tri+i]];
		b=g[v[3*tri+(i+1)%3]];
		c.x = x - a.x; c.y = y - a.y;
		d.x = x - b.x; d.y = y - b.y;
		dist=((b.x-a.x)*(c.y)-(b.y-a.y)*(c.x))/(length(sub(b,a)));
		if (dist<0.0) 
			dist=-dist;

		if (dist <= tol){
			dot1= dot(sub(b,a),c);	
			dot2= dot(sub(a,b),d);	
			if ((min>dist) && (dot1>=0) && (dot2>=0)){
				min=dist;
				mini = i;
			}
		}
	}
	return mini;
}



/*
 * draws the picture 
 */
void draw_control_poly()
{
  int i;

  GLint viewport[4];
  glGetIntegerv (GL_VIEWPORT, viewport);


  glPushMatrix();
  glTranslatef(0.0,0.0,-10.0);
  glScalef(10.0/viewport[3],10.0/viewport[3],10.0/viewport[3]);
  glTranslatef(-0.5*viewport[2],-0.5*viewport[3],0.0);

  
	for(i=0; i < numOfTriangles; i++){
		glColor3f(0.0,1.0,1.0);
		glBegin(GL_TRIANGLES);
			glVertex3f(g[v[3*i  ]].x,g[v[3*i  ]].y, g[v[3*i  ]].z);
			glVertex3f(g[v[3*i+1]].x,g[v[3*i+1]].y, g[v[3*i+1]].z);
			glVertex3f(g[v[3*i+2]].x,g[v[3*i+2]].y, g[v[3*i+2]].z);
		glEnd();

		glColor3f(0.0,0.0,1.0);
		glBegin(GL_LINE_LOOP);
			glVertex3f(g[v[3*i  ]].x,g[v[3*i  ]].y, g[v[3*i  ]].z);
			glVertex3f(g[v[3*i+1]].x,g[v[3*i+1]].y, g[v[3*i+1]].z);
			glVertex3f(g[v[3*i+2]].x,g[v[3*i+2]].y, g[v[3*i+2]].z);
		glEnd();
	}


	//Draws the vertices//
	glBegin(GL_QUADS);
	for (i=0; i<numOfVertices; i++){
		if (i==movingVertex)
			glColor3f(1.0,1.0,0.3);
		else
			glColor3f(1.0,0.0,0.0);

		glVertex3f(g[i].x-tol,g[i].y-tol,0.0);
		glVertex3f(g[i].x-tol,g[i].y+tol,0.0);
		glVertex3f(g[i].x+tol,g[i].y+tol,0.0);
		glVertex3f(g[i].x+tol,g[i].y-tol,0.0);
	}
	glEnd();


	//Draws the ball//
	if(cornerSelect != -1)
		baryBall();
	


  glPopMatrix();

}

/*
 * display the 
 */
void display(void)
{
	glClearColor(1.0,1.0,1.0, 1.0);
  glClear(GL_COLOR_BUFFER_BIT);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  draw_control_poly();
  glFlush();
  glutSwapBuffers();
}

/*
 * Handles the reshaping of the screen
 */
void reshape(int w, int h)
{
   glViewport (0, 0, (GLsizei) w, (GLsizei) h);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluPerspective (2*atan(0.5)/3.14*180,  (GLfloat) w/(GLfloat) h, 1.0, 100.0);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
}


/*
 * Handles the mouse clicks
 */

void mouse_click(int button, int state, int x, int y){
   GLint viewport[4]; //  viewport[3] is height of window in pixels  
   GLint flippedy;    //  OpenGL y coordinate position  
   int p = -1,i,clickedVertex, found = -1,j;
   vertex normal, temp;
   normal.x = 0; normal.y=0; normal.z = 1;

      switch (button){
	case GLUT_LEFT_BUTTON:
		if (state == GLUT_DOWN){
			glGetIntegerv (GL_VIEWPORT, viewport);
		   	flippedy = viewport[3] - (GLint) y - 1;
		   	clickedVertex=vertex_near_point(x,flippedy);
		   	if (clickedVertex>=0){
				temp.x =x; temp.y = flippedy; temp.z = 0;
				for(i=0; i <numOfTriangles;i++){
					if (counterClockwiseOrient(sub(g[v[(3*i)]],temp),sub(g[v[(3*i)+1]],temp)) && 
					    counterClockwiseOrient(sub(g[v[(3*i+1)]],temp),sub(g[v[(3*i)+2]],temp)) &&
					    counterClockwiseOrient(sub(g[v[(3*i)+2]],temp),sub(g[v[(3*i)]],temp))){
						for(j=0;j<3;j++){
							if(v[3*i+j] == clickedVertex){
								cornerSelect =  3*i +j;
								printf("selected corner %d\n",cornerSelect);
							}
						}
					}
				}

		       		//movingVertex=clickedVertex;
		   	} else{
				
				for(i = 0; i < numOfTriangles && p == -1; i++){
					p=edge_near_point(i,x,flippedy);
				}
				i--;	//because of for loop
		        	if (p != -1){
					insert_vertex(x,flippedy);

				//worry about orientation later//

				//	if(counterClockwiseOrient(sub(g[numOfVertices-1],g[3*i+p]),sub(g[numOfVertices-1],g[nOpt(3*i +p)])))
				insert_triangle(numOfVertices-1,v[3*i+p],v[3*i+(p+1)%3]);
				//	else
				//		insert_triangle(numOfVertices-1,v[3*i+(p+1)%3],v[3*i+p]);

						
					o[3*(numOfTriangles-1)] = (3*i+(p+2)%3);
					o[3*i+(p+2)%3] = 3*(numOfTriangles-1);
					movingVertex=numOfVertices -1;

			   	}
		   	}
		   	glutPostRedisplay();
	   	}
	
		if (state == GLUT_UP) {
			i=0;
			//code to see if dropped on a existing vertex //
			while(i<numOfVertices-1 && found == -1){
				if(equals(g[i],g[numOfVertices-1], tol)){
printf("found a matching vertex\n");
					v[(numOfTriangles-1)*3] = i;
					numOfVertices--;
					found = 1;
				}
			i++;
			}

			if(found == 1){
				for(i=0;i< 3*(numOfTriangles-1);i++){
					if(v[3*(numOfTriangles-1)] == v[i]){
						if(v[nOpt(3*(numOfTriangles-1))] == v[nOpt(i)]){
							o[pOpt(3*(numOfTriangles-1))] = pOpt(i);
							o[pOpt(i)] = pOpt(3*(numOfTriangles-1));
						}else if (v[nOpt(3*(numOfTriangles-1))] == v[pOpt(i)]){
							o[pOpt(3*(numOfTriangles-1))] = nOpt(i);
							o[nOpt(i)] = pOpt(3*(numOfTriangles-1));
						}else if (v[pOpt(3*(numOfTriangles-1))] == v[pOpt(i)]){
							o[nOpt(3*(numOfTriangles-1))] = nOpt(i);
							o[nOpt(i)] = nOpt(3*(numOfTriangles-1));
						}else if (v[pOpt(3*(numOfTriangles-1))] == v[nOpt(i)]){
							o[nOpt(3*(numOfTriangles-1))] = pOpt(i);
							o[pOpt(i)] = nOpt(3*(numOfTriangles-1));
						}
					}
				}
			}

			
			movingVertex=-1;
			glutPostRedisplay();
		}
       
	        break;

	default:
		break;
   }
   
}  


/*
 * Handles the mouse drags
 */

void mouse_drag(int x, int y)
{

  GLint viewport[4];
  GLint flippedy;
  if (movingVertex >= 0)
    {
      glGetIntegerv (GL_VIEWPORT, viewport);
      flippedy = viewport[3] - (GLint) y - 1;

			g[movingVertex].x=x;
			g[movingVertex].y=flippedy;

      glutPostRedisplay();
    }
}

/*
 * handles the mouse leave and entering the window
 */
void mouse_leave(int state){
	switch(state){
	case GLUT_LEFT:		
		removeVertex = 1;
		break;
	case GLUT_ENTERED:	
		removeVertex = -1;
		break;
	}
}







//handles keyboard calls
void keyboard(unsigned char key, int x, int y){ 

	switch(key){
	
	case 27:  // ESC 
		print_vertex();
		print_triangles();
		exit(0);
	case 's':
		saveFile();
		break;
	case 'a':
		readFile();
		break;
	case 'u':
		remove_Triangle();
		break;
	case 'o':
		if(cornerSelect !=-1)
			cornerSelect = oOpt(cornerSelect);
		break;
	case 'n':
		if(cornerSelect !=-1)
			cornerSelect = nOpt(cornerSelect);
		break;
	case 'p':
		if(cornerSelect !=-1)
			cornerSelect = pOpt(cornerSelect);
		break;
	case 'l':
		if(cornerSelect !=-1)
			cornerSelect = lOpt(cornerSelect);
		break;
	case 'r':
		if(cornerSelect !=-1)
			cornerSelect = rOpt(cornerSelect);
		break;
	case '1':
		print_oCorners();
		break;
	default:  break;
	}
	glutPostRedisplay();
}




/*
 * starts the program up
 */
int main(int argc, char** argv){	
	int i;
	insert_vertex(50,50);
	insert_vertex(200, 100);
	insert_vertex(100, 100);

	insert_triangle(0,1,2);

	for(i=0;i< 3 * MAXTRIANGLES; i++)
		o[i] = -1;



	glutInit(&argc, argv);
	glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB); /* turn on double buffering */
	glutInitWindowSize (500, 500); 
	glutInitWindowPosition (100, 100);
	glutCreateWindow (argv[0]);
	glutDisplayFunc(display); 
	glutReshapeFunc(reshape);
	glutMouseFunc(mouse_click);
	glutMotionFunc(mouse_drag);
	glutEntryFunc(mouse_leave);
	glutKeyboardFunc(keyboard);

	glutMainLoop();
	return EXIT_SUCCESS;
}

