//
//	Redesigned vectorMath program for Project 2
//	Implemented z diminsion where needed in the 
//	code
//

#include "vectorMath.h"


/*
 *	Finds the dot product of Two vectors
 */
float dot(vertex this, vertex that){
	return (this.x*that.x + this.y*that.y + this.z*that.z);
}

/*
 *	Finds the cross product of Two vectors
 */
vertex cross(vertex this, vertex that){
	vertex temp;
	temp.x = this.y*that.z - this.z*that.y;
	temp.y = this.z*that.x - this.x*that.z;
	temp.z = this.x*that.y - this.y*that.x;
	return temp;
}

/*
 *	Subtracts Two vectors and returns the new vector
 */
vertex sub(vertex this, vertex that){
	vertex temp;
	temp.x = this.x - that.x;
	temp.y = this.y - that.y;
	temp.z = this.z - that.z;
	return temp;
}

/*
 *	Adds Two vectors and returns the new vector
 */
vertex add(vertex this, vertex that){
	vertex temp;
	temp.x = this.x + that.x;
	temp.y = this.y + that.y;
	temp.z = this.z + that.z;
	return temp;
}

/*
 *	Multiply Two vectors and returns the new vector
 */
vertex multiply(vertex this, int scaler){
	vertex temp;
	temp.x = this.x*scaler;
	temp.y = this.y*scaler;
	temp.z = this.z*scaler;
	return temp;
}

/*
 *	divide Two vectors and returns the new vector
 */
vertex divide(vertex this, int scaler){
	vertex temp;
	temp.x = this.x/scaler;
	temp.y = this.y/scaler;
	temp.z = this.z/scaler;
	return temp;
}

/*
 *	Finds the length of Two vectors
 */
float length(vertex this){
	return (float)(sqrt(this.x*this.x + this.y*this.y + this.z*this.z));
}

/*
 *	Finds the unit vector of a given vector
 */
vertex unit(vertex this){
	float len = length(this);
	vertex temp;
	if(len != 0){
		temp = divide(this, len);
	}else{
		temp.x = 0;
		temp.y = 0;
		temp.z = 0;
	}
	return temp;
}

/*
 *	Compares if Two vectors are equal within a given tolerance
 *	of each other.  Z diminsion is not needed because no move-
 *	ment is done in that disminsion manually.
 *
int equals(vertex this, vertex that,int TOL){
	return    (this.x <= that.x + TOL && this.x >= that.x - TOL 
		&& this.y <= that.y + TOL && this.y >= that.y - TOL
	      //&& this.z <= that.z + TOL && this.z >= that.z - TOL
		   );
}
*/

/*
 *	Finds the orthagonal vector to the left of a given vector.
 *	Z diminsion not needed because only called this function in
 *	2D mode
 */
vertex left(vertex this){
	vertex temp;
	temp.x = this.y; temp.y =this.x;	//reverse x and y then change sign
	temp.z = 0;
	if(this.x >= 0 && this.y >= 0) //+ +
		temp.x = -(temp.x);
	else if(this.x < 0 && this.y >= 0) // - +
		temp.y = -(temp.y);
	else if(this.x < 0 && this.y < 0) // - -
		temp.x = -(temp.x);
	else if(this.x >= 0 && this.y < 0) // + -
		temp.y = -(temp.y);
	return temp;

}

/*
 *	Prints the string repesentation of a vector
 */
void toString(vertex this){
	printf(" x:%f, y:%f, z:%f \n", this.x, this.y, this.z);
}

int counterClockwiseOrient(vertex a, vertex b){
	vertex normal;
	normal.x =0.0; normal.y = 0.0; normal.z = 1.0;
 return (dot(cross(a,b),normal) > 0);
}


